//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Import/ImportDataUtil.cpp
//! @brief     Implements ImportDataUtil namespace
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Import/ImportDataUtil.h"
#include "Device/Data/Datafield.h"
#include "GUI/Model/Device/RealItem.h"
#include "GUI/Support/IO/ImportDataInfo.h"
#include "GUI/View/Loaders/QREDataLoader.h"
#include <QApplication>
#include <QMessageBox>

namespace {

const QString filter_string_ba = "Intensity File (*.int *.gz *.tif *.tiff *.txt *.csv);;"
                                 "Other (*.*)";
const QString filter_string_ascii = "Intensity File (*.int *.int.gz *.txt *.csv *.dat *.ascii);;"
                                    "Ascii column-wise data (*.*)";

//! Imports for 2D or 1D data, using given loader.
//! Allocates Datafield, and returns owning pointer.
//! Currently used for all 2D, and some 1D data files.

std::unique_ptr<Datafield> ImportKnownData(const QString& fileName,
                                           IO::Filetype2D ftype = IO::any2D)
{
    // Try to use the canonical tools for importing data
    std::unique_ptr<Datafield> result;
    try {
        result.reset(IO::readData2D(fileName.toStdString(), ftype));
    } catch (std::exception& ex) {
        QString message = QString("Error while trying to read file\n\n'%1'\n\n%2")
                              .arg(fileName)
                              .arg(QString::fromStdString(std::string(ex.what())));
        QMessageBox::warning(nullptr, "IO Problem", message);
    }
    return result;
}

} // namespace


std::unique_ptr<Datafield> GUI::View::ImportDataUtil::Import2dData(const QString& fileName,
                                                                   IO::Filetype2D ftype)
{
    return ImportKnownData(fileName, ftype);
}

QString GUI::View::ImportDataUtil::Import1dData(RealItem* realItem,
                                                const AbstractDataLoader* selectedLoader)
{
    const QString fileName = realItem->nativeFileName();
    const std::string fileNameStdString = fileName.toStdString();

    if (selectedLoader == nullptr) {
        IO::Filetype1D ftype = IO::filename2type1D(fileNameStdString);
        if (IO::isCompressed(fileNameStdString) || ftype == IO::bornagain1D) {
            try {
                ImportDataInfo info(ImportKnownData(fileName), Coords::QSPACE);
                if (info) {
                    realItem->setImportData(std::move(info));
                    return QString();
                }
            } catch (std::exception& ex) {
                // Try with CSV import ? TODO check the logic, simplify
                if (ftype != IO::bornagain1D || IO::isCompressed(fileNameStdString))
                    // import is not possible
                    return QString::fromLatin1(ex.what());
            }
        }
    }

    // -- try with selected CSV loader. If none selected, try with QRE loader

    QFile file(fileName);
    const bool ok = file.open(QFile::ReadOnly | QIODevice::Text);
    if (!ok)
        return "File could not be opened.";

    QByteArray fileContent = file.readAll();
    file.close();

    if (fileContent.isEmpty())
        return "The imported file is empty.";

    if (IO::isCompressed(fileNameStdString)) {
        // #baimport implement decompress
    }

    AbstractDataLoader* loader = nullptr;
    if (selectedLoader == nullptr)
        loader = new QREDataLoader();
    else
        loader = selectedLoader->clone();
    loader->initWithDefaultImportSettings();
    loader->setRealDataItem(realItem);
    realItem->setDataLoader(loader);
    QApplication::setOverrideCursor(Qt::WaitCursor);
    loader->setFileContents(fileContent);
    loader->guessSettings();
    loader->processContents();
    QApplication::restoreOverrideCursor();

    return QString();
}
